# Generated by roxygen2: do not edit by hand

S3method("[",TopDom)
S3method("[",TopDomData)
S3method(as_tibble,TopDomOverlapScores)
S3method(countsPerRegion,TopDomData)
S3method(dim,TopDom)
S3method(dim,TopDomData)
S3method(ggCountHeatmap,TopDomData)
S3method(image,TopDomData)
S3method(print,TopDom)
S3method(print,TopDomData)
S3method(print,TopDomOverlapScores)
S3method(subsetByRegion,TopDom)
S3method(subsetByRegion,TopDomData)
export(TopDom)
export(countsPerRegion)
export(ggCountHeatmap)
export(ggDomain)
export(ggDomainLabel)
export(legacy)
export(overlapScores)
export(readHiC)
export(subsetByRegion)
importFrom(ggplot2,
  aes,
  annotation_custom,
  coord_fixed,
  geom_raster,
  geom_segment,
  ggplot,
  scale_fill_gradient2,
  theme_void
)
importFrom(graphics,image)
importFrom(grid,
  gpar,
  textGrob
)
importFrom(matrixStats,colSums2)
importFrom(reshape2,melt)
importFrom(stats,wilcox.test)
importFrom(tibble,
  as_tibble,
  tibble
)
importFrom(utils,
  capture.output,
  file_test,
  read.table,
  str,
  write.table
)
